@@ -46,7 +46,7 @@ def lombscargle(x,
     freqs : array_like
         Angular frequencies for output periodogram.
     precenter : bool, optional
-        Pre-center amplitudes by subtracting the mean.
+        Pre-center measurement values by subtracting the mean.
     normalize : bool, optional
         Compute normalized periodogram.
 
@@ -94,6 +94,7 @@ def lombscargle(x,
     Examples
     --------
     >>> import matplotlib.pyplot as plt
+    >>> rng = np.random.default_rng()
 
     First define some input parameters for the signal:
 
@@ -102,11 +103,11 @@ def lombscargle(x,
     >>> phi = 0.5 * np.pi
     >>> nin = 1000
     >>> nout = 100000
-    >>> frac_points = 0.9 # Fraction of points to select
+    >>> frac_points = 0.9  # Fraction of points to select
 
     Randomly select a fraction of an array with timesteps:
 
-    >>> r = np.random.rand(nin)
+    >>> r = rng.standard_normal(nin)
     >>> x = np.linspace(0.01, 10*np.pi, nin)
     >>> x = x[r >= frac_points]
 
@@ -135,7 +136,6 @@ def lombscargle(x,
     >>> plt.show()
 
     """
-
     x = np.asarray(x, dtype=np.float64)
     y = np.asarray(y, dtype=np.float64)
     freqs = np.asarray(freqs, dtype=np.float64)
@@ -216,7 +216,7 @@ def periodogram(x, fs=1.0, window='boxcar', nfft=None, detrend='constant',
     --------
     >>> from scipy import signal
     >>> import matplotlib.pyplot as plt
-    >>> np.random.seed(1234)
+    >>> rng = np.random.default_rng()
 
     Generate a test signal, a 2 Vrms sine wave at 1234 Hz, corrupted by
     0.001 V**2/Hz of white noise sampled at 10 kHz.
@@ -228,7 +228,7 @@ def periodogram(x, fs=1.0, window='boxcar', nfft=None, detrend='constant',
     >>> noise_power = 0.001 * fs / 2
     >>> time = np.arange(N) / fs
     >>> x = amp*np.sin(2*np.pi*freq*time)
-    >>> x += np.random.normal(scale=np.sqrt(noise_power), size=time.shape)
+    >>> x += rng.normal(scale=np.sqrt(noise_power), size=time.shape)
 
     Compute and plot the power spectral density.
 
@@ -243,7 +243,7 @@ def periodogram(x, fs=1.0, window='boxcar', nfft=None, detrend='constant',
     peak, we can recover the noise power on the signal.
 
     >>> np.mean(Pxx_den[25000:])
-    0.00099728892368242854
+    0.000985320699252543
 
     Now compute and plot the power spectrum.
 
@@ -384,7 +384,7 @@ def welch(x, fs=1.0, window='hann', nperseg=None, noverlap=None, nfft=None,
     --------
     >>> from scipy import signal
     >>> import matplotlib.pyplot as plt
-    >>> np.random.seed(1234)
+    >>> rng = np.random.default_rng()
 
     Generate a test signal, a 2 Vrms sine wave at 1234 Hz, corrupted by
     0.001 V**2/Hz of white noise sampled at 10 kHz.
@@ -396,7 +396,7 @@ def welch(x, fs=1.0, window='hann', nperseg=None, noverlap=None, nfft=None,
     >>> noise_power = 0.001 * fs / 2
     >>> time = np.arange(N) / fs
     >>> x = amp*np.sin(2*np.pi*freq*time)
-    >>> x += np.random.normal(scale=np.sqrt(noise_power), size=time.shape)
+    >>> x += rng.normal(scale=np.sqrt(noise_power), size=time.shape)
 
     Compute and plot the power spectral density.
 
@@ -445,7 +445,6 @@ def welch(x, fs=1.0, window='hann', nperseg=None, noverlap=None, nfft=None,
     >>> plt.show()
 
     """
-
     freqs, Pxx = csd(x, x, fs=fs, window=window, nperseg=nperseg,
                      noverlap=noverlap, nfft=nfft, detrend=detrend,
                      return_onesided=return_onesided, scaling=scaling,
@@ -458,8 +457,7 @@ def csd(x, y, fs=1.0, window='hann', nperseg=None, noverlap=None, nfft=None,
         detrend='constant', return_onesided=True, scaling='density',
         axis=-1, average='mean'):
     r"""
-    Estimate the cross power spectral density, Pxy, using Welch's
-    method.
+    Estimate the cross power spectral density, Pxy, using Welch's method.
 
     Parameters
     ----------
@@ -506,7 +504,9 @@ def csd(x, y, fs=1.0, window='hann', nperseg=None, noverlap=None, nfft=None,
         Axis along which the CSD is computed for both inputs; the
         default is over the last axis (i.e. ``axis=-1``).
     average : { 'mean', 'median' }, optional
-        Method to use when averaging periodograms. Defaults to 'mean'.
+        Method to use when averaging periodograms. If the spectrum is
+        complex, the average is computed separately for the real and
+        imaginary parts. Defaults to 'mean'.
 
         .. versionadded:: 1.2.0
 
@@ -526,7 +526,7 @@ def csd(x, y, fs=1.0, window='hann', nperseg=None, noverlap=None, nfft=None,
     coherence: Magnitude squared coherence by Welch's method.
 
     Notes
-    --------
+    -----
     By convention, Pxy is computed with the conjugate FFT of X
     multiplied by the FFT of Y.
 
@@ -554,6 +554,7 @@ def csd(x, y, fs=1.0, window='hann', nperseg=None, noverlap=None, nfft=None,
     --------
     >>> from scipy import signal
     >>> import matplotlib.pyplot as plt
+    >>> rng = np.random.default_rng()
 
     Generate two test signals with some common features.
 
@@ -564,10 +565,10 @@ def csd(x, y, fs=1.0, window='hann', nperseg=None, noverlap=None, nfft=None,
     >>> noise_power = 0.001 * fs / 2
     >>> time = np.arange(N) / fs
     >>> b, a = signal.butter(2, 0.25, 'low')
-    >>> x = np.random.normal(scale=np.sqrt(noise_power), size=time.shape)
+    >>> x = rng.normal(scale=np.sqrt(noise_power), size=time.shape)
     >>> y = signal.lfilter(b, a, x)
     >>> x += amp*np.sin(2*np.pi*freq*time)
-    >>> y += np.random.normal(scale=0.1*np.sqrt(noise_power), size=time.shape)
+    >>> y += rng.normal(scale=0.1*np.sqrt(noise_power), size=time.shape)
 
     Compute and plot the magnitude of the cross spectral density.
 
@@ -576,8 +577,8 @@ def csd(x, y, fs=1.0, window='hann', nperseg=None, noverlap=None, nfft=None,
     >>> plt.xlabel('frequency [Hz]')
     >>> plt.ylabel('CSD [V**2/Hz]')
     >>> plt.show()
-    """
 
+    """
     freqs, _, Pxy = _spectral_helper(x, y, fs, window, nperseg, noverlap, nfft,
                                      detrend, return_onesided, scaling, axis,
                                      mode='psd')
@@ -586,7 +587,13 @@ def csd(x, y, fs=1.0, window='hann', nperseg=None, noverlap=None, nfft=None,
     if len(Pxy.shape) >= 2 and Pxy.size > 0:
         if Pxy.shape[-1] > 1:
             if average == 'median':
-                Pxy = np.median(Pxy, axis=-1) / _median_bias(Pxy.shape[-1])
+                # np.median must be passed real arrays for the desired result
+                if np.iscomplexobj(Pxy):
+                    Pxy = (np.median(np.real(Pxy), axis=-1)
+                           + 1j * np.median(np.imag(Pxy), axis=-1))
+                    Pxy /= _median_bias(Pxy.shape[-1])
+                else:
+                    Pxy = np.median(Pxy, axis=-1) / _median_bias(Pxy.shape[-1])
             elif average == 'mean':
                 Pxy = Pxy.mean(axis=-1)
             else:
@@ -601,8 +608,7 @@ def csd(x, y, fs=1.0, window='hann', nperseg=None, noverlap=None, nfft=None,
 def spectrogram(x, fs=1.0, window=('tukey', .25), nperseg=None, noverlap=None,
                 nfft=None, detrend='constant', return_onesided=True,
                 scaling='density', axis=-1, mode='psd'):
-    """
-    Compute a spectrogram with consecutive Fourier transforms.
+    """Compute a spectrogram with consecutive Fourier transforms.
 
     Spectrograms can be used as a way of visualizing the change of a
     nonstationary signal's frequency content over time.
@@ -696,6 +702,7 @@ def spectrogram(x, fs=1.0, window=('tukey', .25), nperseg=None, noverlap=None,
     >>> from scipy import signal
     >>> from scipy.fft import fftshift
     >>> import matplotlib.pyplot as plt
+    >>> rng = np.random.default_rng()
 
     Generate a test signal, a 2 Vrms sine wave whose frequency is slowly
     modulated around 3kHz, corrupted by white noise of exponentially
@@ -708,7 +715,7 @@ def spectrogram(x, fs=1.0, window=('tukey', .25), nperseg=None, noverlap=None,
     >>> time = np.arange(N) / float(fs)
     >>> mod = 500*np.cos(2*np.pi*0.25*time)
     >>> carrier = amp * np.sin(2*np.pi*3e3*time + mod)
-    >>> noise = np.random.normal(scale=np.sqrt(noise_power), size=time.shape)
+    >>> noise = rng.normal(scale=np.sqrt(noise_power), size=time.shape)
     >>> noise *= np.exp(-time/5)
     >>> x = carrier + noise
 
@@ -727,6 +734,7 @@ def spectrogram(x, fs=1.0, window=('tukey', .25), nperseg=None, noverlap=None,
     >>> plt.ylabel('Frequency [Hz]')
     >>> plt.xlabel('Time [sec]')
     >>> plt.show()
+
     """
     modelist = ['psd', 'complex', 'magnitude', 'angle', 'phase']
     if mode not in modelist:
@@ -769,8 +777,7 @@ def spectrogram(x, fs=1.0, window=('tukey', .25), nperseg=None, noverlap=None,
 
 
 def check_COLA(window, nperseg, noverlap, tol=1e-10):
-    r"""
-    Check whether the Constant OverLap Add (COLA) constraint is met
+    r"""Check whether the Constant OverLap Add (COLA) constraint is met.
 
     Parameters
     ----------
@@ -837,33 +844,32 @@ def check_COLA(window, nperseg, noverlap, tol=1e-10):
 
     Confirm COLA condition for rectangular window of 75% (3/4) overlap:
 
-    >>> signal.check_COLA(signal.boxcar(100), 100, 75)
+    >>> signal.check_COLA(signal.windows.boxcar(100), 100, 75)
     True
 
     COLA is not true for 25% (1/4) overlap, though:
 
-    >>> signal.check_COLA(signal.boxcar(100), 100, 25)
+    >>> signal.check_COLA(signal.windows.boxcar(100), 100, 25)
     False
 
     "Symmetrical" Hann window (for filter design) is not COLA:
 
-    >>> signal.check_COLA(signal.hann(120, sym=True), 120, 60)
+    >>> signal.check_COLA(signal.windows.hann(120, sym=True), 120, 60)
     False
 
     "Periodic" or "DFT-even" Hann window (for FFT analysis) is COLA for
     overlap of 1/2, 2/3, 3/4, etc.:
 
-    >>> signal.check_COLA(signal.hann(120, sym=False), 120, 60)
+    >>> signal.check_COLA(signal.windows.hann(120, sym=False), 120, 60)
     True
 
-    >>> signal.check_COLA(signal.hann(120, sym=False), 120, 80)
+    >>> signal.check_COLA(signal.windows.hann(120, sym=False), 120, 80)
     True
 
-    >>> signal.check_COLA(signal.hann(120, sym=False), 120, 90)
+    >>> signal.check_COLA(signal.windows.hann(120, sym=False), 120, 90)
     True
 
     """
-
     nperseg = int(nperseg)
 
     if nperseg < 1:
@@ -893,8 +899,7 @@ def check_COLA(window, nperseg, noverlap, tol=1e-10):
 
 
 def check_NOLA(window, nperseg, noverlap, tol=1e-10):
-    r"""
-    Check whether the Nonzero Overlap Add (NOLA) constraint is met
+    r"""Check whether the Nonzero Overlap Add (NOLA) constraint is met.
 
     Parameters
     ----------
@@ -958,17 +963,17 @@ def check_NOLA(window, nperseg, noverlap, tol=1e-10):
 
     Confirm NOLA condition for rectangular window of 75% (3/4) overlap:
 
-    >>> signal.check_NOLA(signal.boxcar(100), 100, 75)
+    >>> signal.check_NOLA(signal.windows.boxcar(100), 100, 75)
     True
 
     NOLA is also true for 25% (1/4) overlap:
 
-    >>> signal.check_NOLA(signal.boxcar(100), 100, 25)
+    >>> signal.check_NOLA(signal.windows.boxcar(100), 100, 25)
     True
 
     "Symmetrical" Hann window (for filter design) is also NOLA:
 
-    >>> signal.check_NOLA(signal.hann(120, sym=True), 120, 60)
+    >>> signal.check_NOLA(signal.windows.hann(120, sym=True), 120, 60)
     True
 
     As long as there is overlap, it takes quite a pathological window to fail
@@ -982,14 +987,14 @@ def check_NOLA(window, nperseg, noverlap, tol=1e-10):
     If there is not enough overlap, a window with zeros at the ends will not
     work:
 
-    >>> signal.check_NOLA(signal.hann(64), 64, 0)
+    >>> signal.check_NOLA(signal.windows.hann(64), 64, 0)
     False
-    >>> signal.check_NOLA(signal.hann(64), 64, 1)
+    >>> signal.check_NOLA(signal.windows.hann(64), 64, 1)
     False
-    >>> signal.check_NOLA(signal.hann(64), 64, 2)
+    >>> signal.check_NOLA(signal.windows.hann(64), 64, 2)
     True
-    """
 
+    """
     nperseg = int(nperseg)
 
     if nperseg < 1:
@@ -1022,8 +1027,7 @@ def check_NOLA(window, nperseg, noverlap, tol=1e-10):
 def stft(x, fs=1.0, window='hann', nperseg=256, noverlap=None, nfft=None,
          detrend=False, return_onesided=True, boundary='zeros', padded=True,
          axis=-1):
-    r"""
-    Compute the Short Time Fourier Transform (STFT).
+    r"""Compute the Short Time Fourier Transform (STFT).
 
     STFTs can be used as a way of quantifying the change of a
     nonstationary signal's frequency and phase content over time.
@@ -1139,6 +1143,7 @@ def stft(x, fs=1.0, window='hann', nperseg=256, noverlap=None, nfft=None,
     --------
     >>> from scipy import signal
     >>> import matplotlib.pyplot as plt
+    >>> rng = np.random.default_rng()
 
     Generate a test signal, a 2 Vrms sine wave whose frequency is slowly
     modulated around 3kHz, corrupted by white noise of exponentially
@@ -1151,8 +1156,8 @@ def stft(x, fs=1.0, window='hann', nperseg=256, noverlap=None, nfft=None,
     >>> time = np.arange(N) / float(fs)
     >>> mod = 500*np.cos(2*np.pi*0.25*time)
     >>> carrier = amp * np.sin(2*np.pi*3e3*time + mod)
-    >>> noise = np.random.normal(scale=np.sqrt(noise_power),
-    ...                          size=time.shape)
+    >>> noise = rng.normal(scale=np.sqrt(noise_power),
+    ...                    size=time.shape)
     >>> noise *= np.exp(-time/5)
     >>> x = carrier + noise
 
@@ -1164,8 +1169,8 @@ def stft(x, fs=1.0, window='hann', nperseg=256, noverlap=None, nfft=None,
     >>> plt.ylabel('Frequency [Hz]')
     >>> plt.xlabel('Time [sec]')
     >>> plt.show()
-    """
 
+    """
     freqs, time, Zxx = _spectral_helper(x, x, fs, window, nperseg, noverlap,
                                         nfft, detrend, return_onesided,
                                         scaling='spectrum', axis=axis,
@@ -1177,8 +1182,7 @@ def stft(x, fs=1.0, window='hann', nperseg=256, noverlap=None, nfft=None,
 
 def istft(Zxx, fs=1.0, window='hann', nperseg=None, noverlap=None, nfft=None,
           input_onesided=True, boundary=True, time_axis=-1, freq_axis=-2):
-    r"""
-    Perform the inverse Short Time Fourier transform (iSTFT).
+    r"""Perform the inverse Short Time Fourier transform (iSTFT).
 
     Parameters
     ----------
@@ -1285,6 +1289,7 @@ def istft(Zxx, fs=1.0, window='hann', nperseg=None, noverlap=None, nfft=None,
     --------
     >>> from scipy import signal
     >>> import matplotlib.pyplot as plt
+    >>> rng = np.random.default_rng()
 
     Generate a test signal, a 2 Vrms sine wave at 50Hz corrupted by
     0.001 V**2/Hz of white noise sampled at 1024 Hz.
@@ -1296,8 +1301,8 @@ def istft(Zxx, fs=1.0, window='hann', nperseg=None, noverlap=None, nfft=None,
     >>> noise_power = 0.001 * fs / 2
     >>> time = np.arange(N) / float(fs)
     >>> carrier = amp * np.sin(2*np.pi*50*time)
-    >>> noise = np.random.normal(scale=np.sqrt(noise_power),
-    ...                          size=time.shape)
+    >>> noise = rng.normal(scale=np.sqrt(noise_power),
+    ...                    size=time.shape)
     >>> x = carrier + noise
 
     Compute the STFT, and plot its magnitude
@@ -1340,7 +1345,6 @@ def istft(Zxx, fs=1.0, window='hann', nperseg=None, noverlap=None, nfft=None,
     >>> plt.show()
 
     """
-
     # Make sure input is an ndarray of appropriate complex dtype
     Zxx = np.asarray(Zxx) + 0j
     freq_axis = int(freq_axis)
@@ -1514,7 +1518,7 @@ def coherence(x, y, fs=1.0, window='hann', nperseg=None, noverlap=None,
     csd: Cross spectral density by Welch's method.
 
     Notes
-    --------
+    -----
     An appropriate amount of overlap will depend on the choice of window
     and on your requirements. For the default Hann window an overlap of
     50% is a reasonable trade off between accurately estimating the
@@ -1536,6 +1540,7 @@ def coherence(x, y, fs=1.0, window='hann', nperseg=None, noverlap=None,
     --------
     >>> from scipy import signal
     >>> import matplotlib.pyplot as plt
+    >>> rng = np.random.default_rng()
 
     Generate two test signals with some common features.
 
@@ -1546,10 +1551,10 @@ def coherence(x, y, fs=1.0, window='hann', nperseg=None, noverlap=None,
     >>> noise_power = 0.001 * fs / 2
     >>> time = np.arange(N) / fs
     >>> b, a = signal.butter(2, 0.25, 'low')
-    >>> x = np.random.normal(scale=np.sqrt(noise_power), size=time.shape)
+    >>> x = rng.normal(scale=np.sqrt(noise_power), size=time.shape)
     >>> y = signal.lfilter(b, a, x)
     >>> x += amp*np.sin(2*np.pi*freq*time)
-    >>> y += np.random.normal(scale=0.1*np.sqrt(noise_power), size=time.shape)
+    >>> y += rng.normal(scale=0.1*np.sqrt(noise_power), size=time.shape)
 
     Compute and plot the coherence.
 
@@ -1558,8 +1563,8 @@ def coherence(x, y, fs=1.0, window='hann', nperseg=None, noverlap=None,
     >>> plt.xlabel('frequency [Hz]')
     >>> plt.ylabel('Coherence')
     >>> plt.show()
-    """
 
+    """
     freqs, Pxx = welch(x, fs=fs, window=window, nperseg=nperseg,
                        noverlap=noverlap, nfft=nfft, detrend=detrend,
                        axis=axis)
@@ -1577,8 +1582,7 @@ def _spectral_helper(x, y, fs=1.0, window='hann', nperseg=None, noverlap=None,
                      nfft=None, detrend='constant', return_onesided=True,
                      scaling='density', axis=-1, mode='psd', boundary=None,
                      padded=False):
-    """
-    Calculate various forms of windowed FFTs for PSD, CSD, etc.
+    """Calculate various forms of windowed FFTs for PSD, CSD, etc.
 
     This is a helper function that implements the commonality between
     the stft, psd, csd, and spectrogram functions. It is not designed to
@@ -1586,7 +1590,7 @@ def _spectral_helper(x, y, fs=1.0, window='hann', nperseg=None, noverlap=None,
     from each window is returned.
 
     Parameters
-    ---------
+    ----------
     x : array_like
         Array or sequence containing the data to be analyzed.
     y : array_like
@@ -1701,8 +1705,8 @@ def _spectral_helper(x, y, fs=1.0, window='hann', nperseg=None, noverlap=None,
         youter.pop(axis)
         try:
             outershape = np.broadcast(np.empty(xouter), np.empty(youter)).shape
-        except ValueError:
-            raise ValueError('x and y cannot be broadcast together.')
+        except ValueError as e:
+            raise ValueError('x and y cannot be broadcast together.') from e
 
     if same_data:
         if x.size == 0:
@@ -1870,7 +1874,7 @@ def _spectral_helper(x, y, fs=1.0, window='hann', nperseg=None, noverlap=None,
 def _fft_helper(x, win, detrend_func, nperseg, noverlap, nfft, sides):
     """
     Calculate windowed FFT, for internal use by
-    scipy.signal._spectral_helper
+    `scipy.signal._spectral_helper`.
 
     This is a helper function that does the main FFT calculation for
     `_spectral helper`. All input validation is performed there, and the
@@ -1951,7 +1955,6 @@ def _triage_segments(window, nperseg, input_length):
         6
         window.
     """
-
     # parse window; if array like, then set nperseg = win.shape
     if isinstance(window, str) or isinstance(window, tuple):
         # if nperseg not specified
@@ -1983,7 +1986,7 @@ def _median_bias(n):
     Returns the bias of the median of a set of periodograms relative to
     the mean.
 
-    See arXiv:gr-qc/0509116 Appendix B for details.
+    See Appendix B from [1]_ for details.
 
     Parameters
     ----------
@@ -1994,6 +1997,13 @@ def _median_bias(n):
     -------
     bias : float
         Calculated bias.
+
+    References
+    ----------
+    .. [1] B. Allen, W.G. Anderson, P.R. Brady, D.A. Brown, J.D.E. Creighton.
+           "FINDCHIRP: an algorithm for detection of gravitational waves from
+           inspiraling compact binaries", Physical Review D 85, 2012,
+           :arxiv:`gr-qc/0509116`
     """
     ii_2 = 2 * np.arange(1., (n-1) // 2 + 1)
     return 1 + np.sum(1. / (ii_2 + 1) - 1. / ii_2)
